/*
    ChibiOS/RT - Copyright (C) 2014 Uladzimir Pylinsky aka barthess

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
*/

#ifndef _ONEWIRE_H_
#define _ONEWIRE_H_

#include "hal.h"

typedef enum {
  onewire_OK = 0,
  onewire_No_Dev,
  onewire_Dev_Lost,
  onewire_CRC_Err,
  onewire_MemCmp_Err
} onewire_error_t;

typedef enum {
  ow1 = 0,
  ow2 = 1
} onewire_dev_t;

typedef struct onewire {
  int32_t         temperature; /* temperature values in millicelsius */
  size_t          dev_on_bus;
  onewire_error_t err_code;
  const onewireConfig * ow_cfg;
} onewired_t;

/*
 * PROTOTYPES
 */
#ifdef __cplusplus
extern "C" {
#endif
  void onewireMeasure(onewired_t * ow);
  onewire_error_t onewireGetErrorCode(onewired_t * ow);
  uint8_t onewireGetDevicesNum(onewired_t * ow);
  int32_t onewireGetTemperature(onewired_t * ow);
#ifdef __cplusplus
}
#endif

#endif /* _ONEWIRE_H_ */
